﻿using System.ComponentModel;
using System.ComponentModel.Composition;
using System.ComponentModel.Composition.Hosting;
using System.Waf.Applications;
using EmailClient.Applications.ViewModels;
using EmailClient.Applications.Views;
using EmailClient.Domain;

namespace EmailClient.Applications.Controllers
{
    /// <summary>
    /// Responsible for the email account management inclusive the wizard.
    /// </summary>
    internal class CreateEmailAccountController : Controller
    {
        private readonly CompositionContainer container;
        private readonly object shellView;
        private readonly EmailClientRoot emailClientRoot;
        private readonly DelegateCommand backCommand;
        private readonly DelegateCommand nextCommand;
        private readonly EmailAccountsViewModel emailAccountsViewModel;
        private readonly CreateEmailAccountViewModel createEmailAccountViewModel;
        
        private Pop3SettingsViewModel pop3SettingsViewModel;
        private ExchangeSettingsViewModel exchangeSettingsViewModel;


        [ImportingConstructor]
        public CreateEmailAccountController(CompositionContainer container, object shellView, EmailClientRoot emailClientRoot)
        {
            this.container = container;
            this.shellView = shellView;
            this.emailClientRoot = emailClientRoot;
            this.emailAccountsViewModel = container.GetExportedValue<EmailAccountsViewModel>();
            AddWeakEventListener(emailAccountsViewModel, EmailAccountsViewModelPropertyChanged);
            this.createEmailAccountViewModel = container.GetExportedValue<CreateEmailAccountViewModel>();

            backCommand = new DelegateCommand(Back, CanBack);
            nextCommand = new DelegateCommand(Next, CanNext);
        }

        
        public void Initialize()
        {
            emailAccountsViewModel.BackCommand = backCommand;
            emailAccountsViewModel.NextCommand = nextCommand;
        }

        public void Run()
        {
            emailAccountsViewModel.ContentView = createEmailAccountViewModel.View;
            emailAccountsViewModel.ShowDialog(shellView);
        }


        // Wizard workflow

        private void Close()
        {
            emailAccountsViewModel.Close();
        }

        private bool CanBack()
        {
            return emailAccountsViewModel.ContentView != createEmailAccountViewModel.View;
        }

        private void Back()
        {
            emailAccountsViewModel.IsLastPage = false;
            emailAccountsViewModel.ContentView = createEmailAccountViewModel.View;
            UpdateCommandsState();
        }

        private bool CanNext() { return emailAccountsViewModel.IsValid; }

        private void Next()
        {
            if (emailAccountsViewModel.ContentView == createEmailAccountViewModel.View)
            {
                if (createEmailAccountViewModel.IsPop3Checked)
                {
                    emailAccountsViewModel.IsLastPage = true;
                    ShowPop3SettingsView();
                }
                else if (createEmailAccountViewModel.IsExchangeChecked)
                {
                    emailAccountsViewModel.IsLastPage = true;
                    ShowExchangeSettingsView();
                }
            }
            else if (pop3SettingsViewModel != null
                && emailAccountsViewModel.ContentView == pop3SettingsViewModel.View)
            {
                SavePop3Settings();
                Close();
            }
            else if (exchangeSettingsViewModel != null
                && emailAccountsViewModel.ContentView == exchangeSettingsViewModel.View)
            {
                SaveExchangeSettings();
                Close();
            }

            UpdateCommandsState();
        }

        private void UpdateCommandsState()
        {
            backCommand.RaiseCanExecuteChanged();
            nextCommand.RaiseCanExecuteChanged();
        }
        
        private void EmailAccountsViewModelPropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            if (e.PropertyName == "IsValid") { UpdateCommandsState(); }
        }

        
        // Show wizard pages

        private void ShowPop3SettingsView()
        {
            Pop3Settings pop3Settings = new Pop3Settings();
            IPop3SettingsView pop3SettingsView = container.GetExportedValue<IPop3SettingsView>();
            pop3SettingsViewModel = new Pop3SettingsViewModel(pop3SettingsView, pop3Settings);
            emailAccountsViewModel.ContentView = pop3SettingsViewModel.View;
        }

        private void ShowExchangeSettingsView()
        {
            ExchangeSettings exchangeSettings = new ExchangeSettings();
            IExchangeSettingsView exchangeSettingsView = container.GetExportedValue<IExchangeSettingsView>();
            exchangeSettingsViewModel = new ExchangeSettingsViewModel(exchangeSettingsView, exchangeSettings);
            emailAccountsViewModel.ContentView = exchangeSettingsViewModel.View;
        }

        private void SavePop3Settings()
        {
            emailClientRoot.AddEmailAccount(pop3SettingsViewModel.Model);
        }

        private void SaveExchangeSettings()
        {
            emailClientRoot.AddEmailAccount(exchangeSettingsViewModel.Model);
        }
    }
}
